--[[

Version 0.01 - Initial release of Concept code to wowace svn

Author : Levva of EU Khadgar 
	
--]]

local L = LibStub("AceLocale-3.0"):GetLocale("RaidInviter")
local AceAddon = LibStub("AceAddon-3.0")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
RaidInviter = AceAddon:NewAddon("RaidInviter", "AceConsole-3.0", "AceEvent-3.0")
local REVISION = tonumber(("$Revision: 14 $"):match("%d+"))

RaidInviter.textures = {}
RaidInviter.borders = {}
RaidInviter.fonts = {}
RaidInviter.sounds = {}
RaidInviter.dialogBackdrop ={
		bgFile = "Interface/DialogFrame/UI-DialogBox-Background",
		edgeFile= "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32,
		insets = { left = 11, right = 12, top = 12, bottom = 11 }
	}
RaidInviter.tooltipBackdrop ={
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile= "Interface/Tooltips/UI-Tooltip-Border",
		tile = true, tileSize = 32, edgeSize = 32,
		insets = { left = 5, right = 5, top = 5, bottom = 5 }
	}

-------------------
-- Config details
-------------------

RaidInviter.defaults = {
	char = {
		message = "Welcome Home!",
		relativeTo = "UIParent",
		relativePoint = "CENTER",
		point = "CENTER",
		texture = "Blizzard",
		msgfont = "Friz Quadrata TT",
		msgfontsize = 24,
		msgfonteffect = "none",
		border = "None",
		barborder = "Blizzard Tooltip",
		xOffset = 0,
		yOffset = 0,
		scale = 1,
		debug = false,
	}
}

-----------------------------------------
-- Initialisation & Startup Routines
-----------------------------------------

function RaidInviter:OnInitialize()
	local AceConfigReg = LibStub("AceConfigRegistry-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")

	self.db = LibStub("AceDB-3.0"):New("RaidInviterDBPC", RaidInviter.defaults, "char")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RaidInviter", self:GetOptions(), {"RaidInviter", "ri"} )
	media.RegisterCallback(self, "LibSharedMedia_Registered")
	
	-- Add the options to blizzard frame (add them backwards so they show up in the proper order
	self.optionsFrame = AceConfigDialog:AddToBlizOptions("RaidInviter","RaidInviter")
	self.optionsFrame[L["About"]] = LibStub("LibAboutPanel").new("RaidInviter", "RaidInviter")
	self.db:RegisterDefaults(self.defaults)
	local version = GetAddOnMetadata("RaidInviter","Version")
	self.version = ("RaidInviter v%s (r%s)"):format(version, REVISION)
	self:Print(self.version.." Loaded.")
end

function RaidInviter:OnDisable()
    -- Called when the addon is disabled
end

function RaidInviter:OnEnable()
	self:LibSharedMedia_Registered()
	self:RegisterEvent("UPDATE_BINDINGS")
	self:RegisterEvent("GUILD_ROSTER_UPDATE")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("CALENDAR_OPEN_EVENT")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
--~ 	self:RegisterEvent("CALENDAR_UPDATE_EVENT")
--~ 	self:RegisterEvent("CALENDAR_UPDATE_EVENT_LIST")
--~ 	self:RegisterEvent("CALENDAR_UPDATE_INVITE_LIST")
--~ 	self:RegisterEvent("CALENDAR_UPDATE_PENDING_INVITES")
end

function RaidInviter:LibSharedMedia_Registered()
	for k, v in pairs(media:List("statusbar")) do
		self.textures[v] = v
	end
	for k, v in pairs(media:List("border")) do
		self.borders[v] = v
	end
	for k, v in pairs(media:List("font")) do
		self.fonts[v] = v
	end
	for k, v in pairs(media:List("sound")) do
		self.sounds[v] = v
	end
end

----------------------
-- Event Routines
----------------------

function RaidInviter:PLAYER_ENTERING_WORLD()
	self.weekday, self.month, self.day, self.year = CalendarGetDate()
	self:CreateInviteFrame()
	self.selectedEvent = {}
	self.selectedEvent.players = {}
end

function RaidInviter:UPDATE_BINDINGS()
--~ 	self:UpdateBindings()
end

function RaidInviter:GUILD_ROSTER_UPDATE()
	self.guildRoster = {}
	local guildMembers = GetNumGuildMembers(true)
	for index = 1, guildMembers do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
		if name then 
			self:DebugPrint("added "..name.." to guild roster list")
			self.guildRoster[name] = {}
			self.guildRoster[name].name = name
			self.guildRoster[name].rank = rank
			self.guildRoster[name].level = level
			self.guildRoster[name].zone = zone
			self.guildRoster[name].note = note or ""
			self.guildRoster[name].officernote = officernote or ""
			self.guildRoster[name].online = online
			self.guildRoster[name].status = status  -- "<AFK>", "<DND>", or ""
			self.guildRoster[name].lastonline = GetGuildRosterLastOnline(index)
		end
	end
	if self.InviteFrame:IsVisible() then
		self:UpdateInviteFrameWithGuildInfo()
	end
end

function RaidInviter:PARTY_MEMBERS_CHANGED()
	-- this fires when someone joins or leaves the raid or players are moved around in the raid
	-- when it fires we need to update the invite frame with their new status
end

function RaidInviter:CALENDAR_OPEN_EVENT()
	local info = UIDropDownMenu_CreateInfo()
	local title, description, creator, eventType, instanceID, ownerFlag, _, weekday, month, day, year, hour, minute = CalendarGetEventInfo()
	local monthOffset, day, index = CalendarGetEventIndex()
	if eventType == CALENDAR_EVENTTYPE_RAID or eventType == CALENDAR_EVENTTYPE_DUNGEON then
		-- TODO setup event info frame with descriptions etc
		local invites = CalendarEventGetNumInvites()
		self:DebugPrint("Calendar_OE : invites :"..invites)
		if not self.todaysEvents then
			self.todaysEvents = {}
		end
		self.todaysEvents[index] = {}
		self.todaysEvents[index].numplayers = invites
		self.todaysEvents[index].players = {}
		for player = 1, invites do
			local name, level, className, classTexture, signupStatus, modStatus = CalendarEventGetInvite(player)
			self:DebugPrint("Calendar_OE : name :"..name.." level:"..level.." className: "..className.." signupstatus:"..signupStatus.." modStatus:"..modStatus)
			--~ CALENDAR_INVITESTATUS_INVITED   = 1
			--~ CALENDAR_INVITESTATUS_ACCEPTED  = 2
			--~ CALENDAR_INVITESTATUS_DECLINED  = 3
			--~ CALENDAR_INVITESTATUS_CONFIRMED = 4
			--~ CALENDAR_INVITESTATUS_OUT       = 5
			--~ CALENDAR_INVITESTATUS_STANDBY   = 6
			--~ 	modStatus = "MODERATOR", "CREATOR", "" 
			self.todaysEvents[index].players[name] = {}
			self.todaysEvents[index].players[name].index = player
			self.todaysEvents[index].players[name].name = name
			self.todaysEvents[index].players[name].level = level
			self.todaysEvents[index].players[name].className = className
			self.todaysEvents[index].players[name].classTexture = classTexture
			self.todaysEvents[index].players[name].signupStatus = signupStatus
			self.todaysEvents[index].players[name].modStatus = modStatus
			self.todaysEvents[index].players[name].inviteStatus = 0 -- need a table with description, colour 0 = none, white, 1 = invited, yellow, 2 = declined, orange, 3 = offline, red, 4 = inraid, green
		end
	end
end

----------------------
-- Frame Routines
----------------------

function RaidInviter:CreateInviteFrame()
	local frameWidth = 600
	self.InviteFrame = CreateFrame("Frame", "RaidInviterBase", UIParent)
	self.InviteFrame:SetScale(self.db.char.scale)
	self.InviteFrame:SetFrameStrata("MEDIUM")
	self.InviteFrame:SetWidth(frameWidth)
	self.InviteFrame:SetHeight(500)
	self.InviteFrame:SetBackdrop(self.tooltipBackdrop)
	self.InviteFrame:SetBackdropColor(1, 1, 1, 0)
	self.InviteFrame:SetMovable(true)
	self.InviteFrame:SetToplevel(true)
	self.InviteFrame:EnableMouse(true)
	self.InviteFrame:RegisterForDrag("LeftButton")
	self.InviteFrame:SetPoint(self.db.char.point, self.db.char.relativeTo, self.db.char.relativePoint, self.db.char.xOffset, self.db.char.yOffset)
	self.InviteFrame:SetScript("OnHide", function() self:RaidInviter_OnHide() end)
	self.InviteFrame:SetScript("OnDragStart", 
		function()
			self.InviteFrame:StartMoving();
		end );
	self.InviteFrame:SetScript("OnDragStop",
		function()
			self.InviteFrame:StopMovingOrSizing();
			self.InviteFrame:SetScript("OnUpdate", nil);
			self:FinishedMoving(self.db.char, self.InviteFrame);
		end );
	self.InviteFrame:SetBackdropColor(0,0,0,1)
	self.InviteFrame.textureFrame = CreateFrame("Frame", nil, self.InviteFrame)
	self.InviteFrame.textureFrame:SetBackdrop(self.tooltipBackdrop)
	self.InviteFrame.textureFrame:SetBackdropColor(1,1,1,1)
	self.InviteFrame.textureFrame:SetWidth(frameWidth)
	self.InviteFrame.textureFrame:SetHeight(12)
	self.InviteFrame.textureFrame:SetPoint("TOP", self.InviteFrame, "TOP", 0, -28)
	self.InviteFrame.textureFrame:Show()
	
	self.InviteFrame.headertext = self.InviteFrame.textureFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
	self.InviteFrame.headertext:ClearAllPoints()
	self.InviteFrame.headertext:SetWidth(frameWidth)
	self.InviteFrame.headertext:SetHeight(12)
	self.InviteFrame.headertext:SetPoint("CENTER", self.InviteFrame, "TOP", 0, -18)
--~ 	self.InviteFrame.headertext:SetFont("Friz Quadrata TT", 24)
	self.InviteFrame.headertext:SetText("Raid Inviter")
	local closeButton = CreateFrame("Button", nil, self.InviteFrame, "UIPanelCloseButton")
	closeButton:SetScript("OnClick", function() self:RaidInviter_OnHide() end)
	closeButton:SetPoint("TOPRIGHT", self.InviteFrame, "TOPRIGHT", 0, -1)
	closeButton:SetScale(1.2)

	self:CreateEventInfoFrame()
	self:CreateEventListDropdown()
	self.InviteFrame.playerList = self:CreatePlayerListFrame()
	self.InviteFrame.raidRoleComposition, self.InviteFrame.raidClassComposition = self:CreateRaidCompositionFrames()
	if self.db.char.debug then
		self.InviteFrame:Show()
	end
end

function RaidInviter:CreateEventInfoFrame()
	if not self.InviteFrame.event then
		self.InviteFrame.event =  CreateFrame("Frame", "RaidInviterEventInfo", self.InviteFrame)
	end
	self.InviteFrame.event:ClearAllPoints()
	self.InviteFrame.event:SetWidth(450)
	self.InviteFrame.event:SetHeight(100)
	self.InviteFrame.event:SetBackdrop(RaidInviter.tooltipBackdrop)
	self.InviteFrame.event:SetPoint("CENTER", self.InviteFrame, "TOP", 0, 50)
	self.InviteFrame.event:Show()
end

function RaidInviter:CreateEventListDropdown()
	if not self.InviteFrame.event.dropdown then
		self.InviteFrame.event.dropdown = CreateFrame("Frame", "RaidInviterEventList", self.InviteFrame.event, "UIDropDownMenuTemplate")
	end
	self.InviteFrame.event.dropdown:SetPoint("CENTER", self.InviteFrame.event, "TOP", 0 , -10)
	UIDropDownMenu_SetWidth(self.InviteFrame.event.dropdown, 200)
	UIDropDownMenu_Initialize(self.InviteFrame.event.dropdown, function(self) RaidInviter:PopulateEventList() end)
	self.InviteFrame.event.dropdown.text = self.InviteFrame.event:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
	self.InviteFrame.event.dropdown.text:SetFont("Friz Quadrata TT", 14)
	self.InviteFrame.event.dropdown.text:SetTextColor(1,1,1,1)
	self.InviteFrame.event.dropdown.text:SetPoint("RIGHT", eventList, "LEFT", 0, 2)
	self.InviteFrame.event.dropdown.text:SetText(L["Select Event"])
	self.InviteFrame.event.dropdown:Show()
end

function RaidInviter:PopulateEventList()
	local info = UIDropDownMenu_CreateInfo()
	local eventsToday = CalendarGetNumDayEvents(0, self.day)
	self:DebugPrint("day : "..self.day.." found "..eventsToday.." events today")
	local uiEvents = 0
	if eventsToday > 0 then
		for index = 1, eventsToday do
			local title, _, _,  _, _, eventType = CalendarGetDayEvent( 0, self.day, index ) 
			self:DebugPrint("found event type:"..eventType.." title : "..title)
			if eventType == 1 then -- CALENDAR_EVENTTYPE_RAID
				info.text = title
				info.value = index
				info.func = function() RaidInviter:EventListItem_OnClick() end
				info.owner = self.InviteFrame.event.dropdown
				info.checked = index == 1
				info.icon = nil
				UIDropDownMenu_AddButton(info)
				uiEvents = uiEvents + 1
			end
		end
	end
	if uiEvents == 0 then
		info.text = "No scheduled events today"
		info.value = -1 
		info.func = nil
		info.owner = self.InviteFrame.event.dropdown
		info.checked = true
		info.icon = nil
		UIDropDownMenu_AddButton(info)
	end
end

function RaidInviter:RaidInviter_OnHide()
	self.InviteFrame:Hide()
end

function RaidInviter:RaidInviter_OnShow()
	self.InviteFrame:Show()
end

function RaidInviter:EventListItem_OnClick()
	CalendarOpenEvent(0, self.day, this.value)	
	UIDropDownMenu_SetSelectedValue(this.owner, this.value)
	local guildRoster = self:LoadGuildRoster()
	local title, hour, minute, _, _, _, texture, _, _, _, difficulty = CalendarGetDayEvent( 0, self.day, this.value )
	self:DebugPrint("setting event to "..this.value.." title:"..title.." time "..hour..":"..minute)
	if not self.todaysEvents then
		self.todaysEvents = {}
	end
	self.todaysEvents[this.value] = {}
	self.todaysEvents[this.value].index = this.value
	self.todaysEvents[this.value].title = title
	self.todaysEvents[this.value].hour = hour
	self.todaysEvents[this.value].minute = minute
	self.todaysEvents[this.value].texture = texture
	self.todaysEvents[this.value].difficulty = difficulty
	self.todaysEvents[this.value].players = {}
	self.selectedEvent = self.todaysEvents[this.value]
end

function RaidInviter:LoadGuildRoster()
	self.guildRoster = {}
	if IsInGuild() then
		GuildRoster()
	end
end

function RaidInviter:CreatePlayerListFrame()
	local playerList = CreateFrame("Frame", "RaidInviterPlayerList", self.InviteFrame)
	return playerList
end

function RaidInviter:CreateRaidCompositionFrames()
	local roleCompostion = CreateFrame("Frame", "RaidInviterRoleComposition", self.InviteFrame)
	local classCompostion = CreateFrame("Frame", "RaidInviterClassComposition", self.InviteFrame)
	return roleComposition, classComposition
end


function RaidInviter:UpdateInviteFrame()

end

function RaidInviter:UpdateInviteFrameWithGuildInfo()
	-- this function will update the raidInvites zone, online & afk/dnd status, 
	for name,player in ipairs(self.selectedEvent.players) do
		self:DebugPrint("updating frame with guild info for "..name)
		if self.guildRoster[name] and self.selectedEvent then
			if self.selectedEvent.players[name] then
				player.guildRank = self.guildRoster[name].rank
				player.note = self.guildRoster[name].note
				player.officernote = self.guildRoster[name].officernote
				player.zone = self.guildRoster[name].zone
				player.online = self.guildRoster[name].online
				player.active = self.guildRoster[name].active
			else
				self:Print("called updateInviteFrameWithGuildInfo and didn't find "..name)
			end
		end
	end
end

-------------------------
-- Utilitiy Functions
-------------------------

function RaidInviter:DebugPrint(msg)
	if self.db.char.debug then
		self:Print(msg)
	end
end